searchData={"items":[{"type":"module","doc":"High-performance JSON Schema validation for Elixir using Rust.\n\nThis library provides a fast and spec-compliant JSON Schema validator\npowered by the Rust `jsonschema` crate. It supports multiple JSON Schema\ndraft versions and provides detailed validation error information.","title":"ExJsonschema","ref":"ExJsonschema.html"},{"type":"module","doc":"# Compile a schema\n    schema = ~s({\"type\": \"object\", \"properties\": {\"name\": {\"type\": \"string\"}}})\n    {:ok, compiled} = ExJsonschema.compile(schema)\n\n    # Validate JSON\n    valid_json = ~s({\"name\": \"John\"})\n    :ok = ExJsonschema.validate(compiled, valid_json)\n\n    invalid_json = ~s({\"name\": 123})\n    {:error, errors} = ExJsonschema.validate(compiled, invalid_json)","title":"Quick Start - ExJsonschema","ref":"ExJsonschema.html#module-quick-start"},{"type":"module","doc":"- Fast validation using Rust\n- Support for JSON Schema draft-07, draft 2019-09, and draft 2020-12\n- Detailed error messages with path information\n- Precompiled binaries for easy installation\n- Zero Rust toolchain required for end users","title":"Features - ExJsonschema","ref":"ExJsonschema.html#module-features"},{"type":"function","doc":"Compiles a JSON Schema string into an optimized validator.","title":"ExJsonschema.compile/1","ref":"ExJsonschema.html#compile/1"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> {:ok, compiled} = ExJsonschema.compile(schema)\n    iex> is_reference(compiled)\n    true\n\n    iex> invalid_schema = ~s({\"type\": \"invalid_type\"})\n    iex> {:error, %ExJsonschema.CompilationError{type: :compilation_error}} = ExJsonschema.compile(invalid_schema)","title":"Examples - ExJsonschema.compile/1","ref":"ExJsonschema.html#compile/1-examples"},{"type":"function","doc":"Compiles a JSON Schema, raising an exception on failure.","title":"ExJsonschema.compile!/1","ref":"ExJsonschema.html#compile!/1"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> compiled = ExJsonschema.compile!(schema)\n    iex> is_reference(compiled)\n    true","title":"Examples - ExJsonschema.compile!/1","ref":"ExJsonschema.html#compile!/1-examples"},{"type":"type","doc":"","title":"ExJsonschema.compiled_schema/0","ref":"ExJsonschema.html#t:compiled_schema/0"},{"type":"type","doc":"","title":"ExJsonschema.json_string/0","ref":"ExJsonschema.html#t:json_string/0"},{"type":"function","doc":"Checks if JSON is valid against a compiled schema without returning error details.\n\nThis is faster than `validate/2` when you only need to know if the JSON is valid.","title":"ExJsonschema.valid?/2","ref":"ExJsonschema.html#valid?/2"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> {:ok, compiled} = ExJsonschema.compile(schema)\n    iex> ExJsonschema.valid?(compiled, ~s(\"hello\"))\n    true\n    iex> ExJsonschema.valid?(compiled, ~s(123))\n    false","title":"Examples - ExJsonschema.valid?/2","ref":"ExJsonschema.html#valid?/2-examples"},{"type":"function","doc":"Validates JSON against a compiled schema.\n\nReturns `:ok` if valid, or `{:error, errors}` with detailed error information.","title":"ExJsonschema.validate/2","ref":"ExJsonschema.html#validate/2"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> {:ok, compiled} = ExJsonschema.compile(schema)\n    iex> ExJsonschema.validate(compiled, ~s(\"hello\"))\n    :ok\n    iex> match?({:error, [%ExJsonschema.ValidationError{} | _]}, ExJsonschema.validate(compiled, ~s(123)))\n    true","title":"Examples - ExJsonschema.validate/2","ref":"ExJsonschema.html#validate/2-examples"},{"type":"function","doc":"Validates JSON against a compiled schema, raising an exception on validation failure.","title":"ExJsonschema.validate!/2","ref":"ExJsonschema.html#validate!/2"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> {:ok, compiled} = ExJsonschema.compile(schema)\n    iex> ExJsonschema.validate!(compiled, ~s(\"hello\"))\n    :ok","title":"Examples - ExJsonschema.validate!/2","ref":"ExJsonschema.html#validate!/2-examples"},{"type":"function","doc":"One-shot validation: compiles schema and validates instance in a single call.\n\nThis is convenient for one-time validations but less efficient for repeated\nvalidations of the same schema.","title":"ExJsonschema.validate_once/2","ref":"ExJsonschema.html#validate_once/2"},{"type":"function","doc":"iex> schema = ~s({\"type\": \"string\"})\n    iex> ExJsonschema.validate_once(schema, ~s(\"hello\"))\n    :ok\n    iex> match?({:error, [%ExJsonschema.ValidationError{} | _]}, ExJsonschema.validate_once(schema, ~s(123)))\n    true","title":"Examples - ExJsonschema.validate_once/2","ref":"ExJsonschema.html#validate_once/2-examples"},{"type":"type","doc":"","title":"ExJsonschema.validation_result/0","ref":"ExJsonschema.html#t:validation_result/0"},{"type":"module","doc":"Represents an error that occurred during JSON Schema compilation.\n\nThis struct provides detailed information about what went wrong during \nschema compilation, including the error type, message, and any additional context.","title":"ExJsonschema.CompilationError","ref":"ExJsonschema.CompilationError.html"},{"type":"function","doc":"Creates a CompilationError from a map returned by the NIF.","title":"ExJsonschema.CompilationError.from_map/1","ref":"ExJsonschema.CompilationError.html#from_map/1"},{"type":"type","doc":"","title":"ExJsonschema.CompilationError.t/0","ref":"ExJsonschema.CompilationError.html#t:t/0"},{"type":"module","doc":"Represents a JSON Schema validation error with detailed path and message information.","title":"ExJsonschema.ValidationError","ref":"ExJsonschema.ValidationError.html"},{"type":"function","doc":"Creates a ValidationError from a map returned by the NIF.","title":"ExJsonschema.ValidationError.from_map/1","ref":"ExJsonschema.ValidationError.html#from_map/1"},{"type":"type","doc":"","title":"ExJsonschema.ValidationError.t/0","ref":"ExJsonschema.ValidationError.html#t:t/0"},{"type":"exception","doc":"Exception raised when validation fails using validate!/2.","title":"ExJsonschema.ValidationError.Exception","ref":"ExJsonschema.ValidationError.Exception.html"},{"type":"function","doc":"Callback implementation for  Exception.message/1 .","title":"ExJsonschema.ValidationError.Exception.message/1","ref":"ExJsonschema.ValidationError.Exception.html#message/1"},{"type":"type","doc":"","title":"ExJsonschema.ValidationError.Exception.t/0","ref":"ExJsonschema.ValidationError.Exception.html#t:t/0"},{"type":"extras","doc":"# ExJsonschema\n\n🚀 **High-performance JSON Schema validation for Elixir using Rust**\n\nExJsonschema is a fast, safe, and spec-compliant JSON Schema validator for Elixir, powered by the battle-tested Rust [`jsonschema`](https://crates.io/crates/jsonschema) crate. It provides an idiomatic Elixir API with detailed error reporting and supports multiple JSON Schema draft versions.","title":"Overview","ref":"readme.html"},{"type":"extras","doc":"- **🔥 High Performance**: Rust-powered validation with zero-copy JSON processing\n- **🛡️ Memory Safe**: No risk of crashing the BEAM VM - all Rust panics are caught\n- **📋 Spec Compliant**: Supports JSON Schema draft-07, draft 2019-09, and draft 2020-12\n- **🔍 Detailed Errors**: Rich error messages with path information and validation context\n- **📦 Zero Dependencies**: Precompiled NIFs mean no Rust toolchain required for end users\n- **🎯 Idiomatic Elixir**: Clean, functional API that feels natural in Elixir","title":"✨ Features - Overview","ref":"readme.html#features"},{"type":"extras","doc":"Add `ex_jsonschema` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_jsonschema, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThat's it! No Rust toolchain required - precompiled binaries are included.","title":"📋 Installation - Overview","ref":"readme.html#installation"},{"type":"extras","doc":"```elixir\n# Compile a schema once\nschema = ~s({\n  \"type\": \"object\",\n  \"properties\": {\n    \"name\": {\"type\": \"string\", \"minLength\": 1},\n    \"age\": {\"type\": \"number\", \"minimum\": 0}\n  },\n  \"required\": [\"name\"]\n})\n\n{:ok, compiled} = ExJsonschema.compile(schema)\n\n# Validate JSON - fast validation\nvalid_json = ~s({\"name\": \"Alice\", \"age\": 30})\n:ok = ExJsonschema.validate(compiled, valid_json)\n\n# Invalid JSON with detailed errors\ninvalid_json = ~s({\"age\": -5})\n{:error, errors} = ExJsonschema.validate(compiled, invalid_json)\n\nEnum.each(errors, fn error ->\n  IO.puts(\"❌ #{error.message} at #{error.instance_path}\")\nend)\n# Output:\n# ❌ \"name\" is a required property at \n# ❌ -5 is less than the minimum of 0 at /age\n```","title":"🚀 Quick Start - Overview","ref":"readme.html#quick-start"},{"type":"extras","doc":"","title":"📖 API Reference - Overview","ref":"readme.html#api-reference"},{"type":"extras","doc":"```elixir\n# Compile a schema\n{:ok, compiled} = ExJsonschema.compile(schema_json)\n\n# Compile with error handling\ncase ExJsonschema.compile(schema_json) do\n  {:ok, compiled} -> \n    # Use compiled schema\n  {:error, %ExJsonschema.CompilationError{} = error} ->\n    IO.puts(\"Schema error: #{error}\")\nend\n\n# Compile and raise on error\ncompiled = ExJsonschema.compile!(schema_json)\n```","title":"Schema Compilation - Overview","ref":"readme.html#schema-compilation"},{"type":"extras","doc":"```elixir\n# Full validation with detailed errors\ncase ExJsonschema.validate(compiled, json) do\n  :ok -> \n    IO.puts(\"Valid!\")\n  {:error, errors} ->\n    Enum.each(errors, &IO.puts(\"Error: #{&1}\"))\nend\n\n# Quick validity check (faster)\nif ExJsonschema.valid?(compiled, json) do\n  IO.puts(\"Valid!\")\nend\n\n# One-shot validation (compile + validate)\nExJsonschema.validate_once(schema_json, instance_json)\n\n# Validation with exceptions\nExJsonschema.validate!(compiled, json)\n```","title":"Validation - Overview","ref":"readme.html#validation"},{"type":"extras","doc":"ExJsonschema provides detailed error information to help you debug validation issues:","title":"🔍 Enhanced Error Handling - Overview","ref":"readme.html#enhanced-error-handling"},{"type":"extras","doc":"```elixir\n# JSON parsing error\ninvalid_json = ~s({\"type\": \"string)  # Missing quote\n{:error, error} = ExJsonschema.compile(invalid_json)\n\nerror.type     # :json_parse_error\nerror.message  # \"Invalid JSON: EOF while parsing a string at line 1 column 16\"\nerror.details  # \"Failed to parse JSON at line 1, column 16\"\n\n# Schema validation error  \ninvalid_schema = ~s({\"type\": \"invalid_type\"})\n{:error, error} = ExJsonschema.compile(invalid_schema)\n\nerror.type     # :compilation_error  \nerror.message  # \"Schema compilation failed\"\nerror.details  # \"\\\"invalid_type\\\" is not valid under any of the schemas listed in the 'anyOf' keyword\"\n```","title":"Schema Compilation Errors - Overview","ref":"readme.html#schema-compilation-errors"},{"type":"extras","doc":"```elixir\nschema = ~s({\n  \"type\": \"object\",\n  \"properties\": {\n    \"user\": {\n      \"type\": \"object\", \n      \"properties\": {\n        \"age\": {\"type\": \"number\", \"minimum\": 0}\n      }\n    }\n  }\n})\n\n{:ok, compiled} = ExJsonschema.compile(schema)\n{:error, [error]} = ExJsonschema.validate(compiled, ~s({\"user\": {\"age\": -5}}))\n\nerror.instance_path  # \"/user/age\"\nerror.schema_path    # \"/properties/user/properties/age/minimum\"  \nerror.message        # \"-5 is less than the minimum of 0\"\n```","title":"Validation Errors - Overview","ref":"readme.html#validation-errors"},{"type":"extras","doc":"For development or if you need to build from source:\n\n```bash\n# Clone the repository\ngit clone https://github.com/your-username/ex_jsonschema.git\ncd ex_jsonschema\n\n# Install dependencies\nmix deps.get\n\n# Force build from source (requires Rust toolchain)\nEX_JSONSCHEMA_BUILD=1 mix compile\n\n# Run tests\nEX_JSONSCHEMA_BUILD=1 mix test\n```","title":"🏗️ Building from Source - Overview","ref":"readme.html#building-from-source"},{"type":"extras","doc":"ExJsonschema is designed for high-performance applications:\n\n- **Compiled schemas**: Compile once, validate many times\n- **Zero-copy**: Direct validation of JSON strings without intermediate parsing\n- **Rust performance**: Orders of magnitude faster than pure Elixir implementations\n- **Memory efficient**: Minimal memory allocation during validation","title":"🎯 Performance - Overview","ref":"readme.html#performance"},{"type":"extras","doc":"ExJsonschema supports multiple JSON Schema draft versions:\n\n- ✅ **Draft 7** (2019)\n- ✅ **Draft 2019-09** \n- ✅ **Draft 2020-12** (latest)\n\nAll core keywords and validation rules are supported, including:\n- Type validation (`string`, `number`, `object`, `array`, etc.)\n- Constraints (`minimum`, `maximum`, `minLength`, `maxLength`, etc.)\n- Object validation (`properties`, `required`, `additionalProperties`, etc.)\n- Array validation (`items`, `minItems`, `maxItems`, etc.)\n- String formats (`email`, `uri`, `date-time`, etc.)\n- Conditional schemas (`if`/`then`/`else`, `allOf`, `anyOf`, `oneOf`)","title":"🤝 JSON Schema Support - Overview","ref":"readme.html#json-schema-support"},{"type":"extras","doc":"","title":"📚 Examples - Overview","ref":"readme.html#examples"},{"type":"extras","doc":"```elixir\nuser_schema = ~s({\n  \"type\": \"object\",\n  \"properties\": {\n    \"email\": {\"type\": \"string\", \"format\": \"email\"},\n    \"username\": {\"type\": \"string\", \"minLength\": 3, \"maxLength\": 20},\n    \"age\": {\"type\": \"integer\", \"minimum\": 13, \"maximum\": 120},\n    \"preferences\": {\n      \"type\": \"object\",\n      \"properties\": {\n        \"newsletter\": {\"type\": \"boolean\"},\n        \"theme\": {\"type\": \"string\", \"enum\": [\"light\", \"dark\"]}\n      }\n    }\n  },\n  \"required\": [\"email\", \"username\", \"age\"]\n})\n\n{:ok, compiled} = ExJsonschema.compile(user_schema)\n\n# Valid user\nuser_data = ~s({\n  \"email\": \"alice@example.com\",\n  \"username\": \"alice_cooper\", \n  \"age\": 25,\n  \"preferences\": {\n    \"newsletter\": true,\n    \"theme\": \"dark\"\n  }\n})\n\n:ok = ExJsonschema.validate(compiled, user_data)\n```","title":"User Registration Validation - Overview","ref":"readme.html#user-registration-validation"},{"type":"extras","doc":"```elixir\napi_response_schema = ~s({\n  \"type\": \"object\",\n  \"properties\": {\n    \"status\": {\"type\": \"string\", \"enum\": [\"success\", \"error\"]},\n    \"data\": {\"type\": \"object\"},\n    \"message\": {\"type\": \"string\"},\n    \"timestamp\": {\"type\": \"string\", \"format\": \"date-time\"}\n  },\n  \"required\": [\"status\", \"timestamp\"],\n  \"if\": {\"properties\": {\"status\": {\"const\": \"error\"}}},\n  \"then\": {\"required\": [\"message\"]},\n  \"else\": {\"required\": [\"data\"]}\n})\n```","title":"API Response Validation - Overview","ref":"readme.html#api-response-validation"},{"type":"extras","doc":"","title":"🔧 Development - Overview","ref":"readme.html#development"},{"type":"extras","doc":"- Elixir 1.12+\n- Erlang/OTP 22+\n- Rust 1.70+ (only for building from source)","title":"Requirements - Overview","ref":"readme.html#requirements"},{"type":"extras","doc":"```bash\nmix test                    # Uses precompiled NIF\nEX_JSONSCHEMA_BUILD=1 mix test  # Builds from source\n```","title":"Running Tests - Overview","ref":"readme.html#running-tests"},{"type":"extras","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","title":"📄 License - Overview","ref":"readme.html#license"},{"type":"extras","doc":"- Built on the excellent [jsonschema](https://crates.io/crates/jsonschema) Rust crate\n- Powered by [Rustler](https://github.com/rusterlium/rustler) for safe Rust-Elixir interop\n- Inspired by the JSON Schema specification and community","title":"🙏 Acknowledgments - Overview","ref":"readme.html#acknowledgments"},{"type":"extras","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.1.0] - 2024-12-17","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"- Initial release of ExJsonschema\n- High-performance JSON Schema validation using Rust `jsonschema` crate v0.20\n- Support for JSON Schema draft-07, draft 2019-09, and draft 2020-12\n- Precompiled NIF binaries for major platforms (no Rust toolchain required)\n- Comprehensive API with multiple validation functions:\n  - `compile/1` and `compile!/1` - Schema compilation\n  - `validate/2` and `validate!/2` - Full validation with detailed errors\n  - `valid?/2` - Fast boolean validation check\n  - `validate_once/2` - One-shot compilation and validation\n- Enhanced error handling with structured `CompilationError` and `ValidationError` types\n- Detailed error messages with JSON path information and validation context\n- Memory-safe NIF implementation with proper panic handling\n- Comprehensive test suite with 27 tests covering all functionality\n- Complete documentation with examples and API reference\n- Zero-dependency installation for end users","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Built with Rustler v0.36 for safe Rust-Elixir interop\n- Uses `rustler_precompiled` v0.8 for precompiled binary distribution\n- Implements proper NIF resource management for compiled schemas\n- Supports multiple architectures: x86_64 and aarch64 for macOS, Linux, and Windows\n- Optimized for performance with compile-once, validate-many pattern\n\n[Unreleased]: https://github.com/hassox/ex_jsonschema/compare/v0.1.0...HEAD\n[0.1.0]: https://github.com/hassox/ex_jsonschema/releases/tag/v0.1.0","title":"Technical Details - Changelog","ref":"changelog.html#technical-details"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}